from fuzzywuzzy import process
import requests


class FuzzySearch:
    json = None
    indexed = None

    def _populate(self, name):
        if not self.json and not self.indexed:
            self.json = requests.get('http://api.steampowered.com/ISteamApps/GetAppList/v0001/').json()
        if not self.indexed:
            self.indexed = {idx: k["name"] for idx, k in enumerate(json["applist"]["apps"]["app"])}

    def getAppID(self, name):
        # populate it
        self._populate(name)

        # extract the one with best match
        tup = process.extractOne(name, dict)

        # return if found else return None
        if tup:
            return self.json["applist"]["apps"]["app"][a[2]]["appid"]
        else:
            return None



import requests, re, math, w3lib.html
from bs4 import BeautifulSoup


class Bundle:
    store_url = "https://store.steampowered.com/bundle/"
    data = {}
    appid = 0

    def _getList(self, text, term, space=None):
        temp = text.find(term)
        if not temp:
            return None
        text = text[temp + len(term) + 5:text.find('<br/>', temp)]
        text = w3lib.html.remove_tags(text)
        if space == "space":
            return text.replace("\t", "").replace("</div", "").replace("\n", "")
        text = text.replace(", ", ",").split(",")
        return text

    '''
        DATA
        App ID:
            sucess
            data:
                name
                bundle_id
                header_image
                bundle_description
                genre (list)
                developer (list)
                publisher (list)
                franchise (list)
                language (list)
                drm 
                categories (list)
                price: (dict)
                    initial
                    final
                    discount_percent
                    initial_formatted
                    final_formatted
                package_item: (list)
                    app_name
                    app_id
                    app_link
                    app_image
                    app_price: (dict)
                        final
                        final_formatted
                    platforms: (dict)
                        windows
                        mac
                        linux
                    categories (list)
    '''

    # returns raw data
    def getRaw(self, appid):
        # caching appid
        self.appid = appid

        cookies = {'birthtime': '28801', 'lastagecheckage': '9-1-1991', 'mature_content': '1',
                   'wants_mature_content': '1'}

        # following steam json format
        data = {}
        data[appid] = {}
        data[appid]['success'] = False
        data[appid]['data'] = {}

        res = requests.get(self.store_url + appid, cookies=cookies)

        # checking if bundle exists
        # if bundle does not exist it redirects to store home, thus we check for any redirects
        if not res.history:
            data[appid]['success'] = True

        # if bundle exists procced with parsing
        if data[appid]['success']:
            details = {}
            soup = BeautifulSoup(res.content, 'html5lib')
            # get name of bundle and bundle_id
            details['name'] = soup.select('.pageheader')[0].text
            details['bundle_id'] = soup.select('.game_area_purchase_game')[0].get('data-ds-bundleid')
            # selecting the correct div
            leftcol = soup.find('div', {'class': 'leftcol'})
            rightcol = soup.find('div', {'class': 'rightcol'})

            # header image
            details['header_image'] = leftcol.select('.package_header')[0].get('src')

            # description
            if leftcol.select('.bundle_description'):
                details['bundle_description'] = leftcol.select('.bundle_description')[0].select('p')[0].text

            # from rightcol
            right_detail = str(rightcol.findAll('div', {'class': 'details_block'})[0])
            details['genres'] = self._getList(right_detail, 'Genre:')
            details['developers'] = self._getList(right_detail, 'Developer:')
            details['publishers'] = self._getList(right_detail, 'Publisher:')
            details['franchise'] = self._getList(right_detail, 'Franchise:')
            details['languages'] = self._getList(right_detail, 'Languages:')
            details['drm'] = self._getList(right_detail, 'DRM:', 'space')

            details['categories'] = []
            for category in rightcol.findAll('div', {'class': 'game_area_details_specs'}):
                dict = {}
                id = re.sub(r'^[\w:\/\.\?]+=', '', category.find('a').get('href'))
                id = int(re.sub(r'\D[\w]+', '', id))
                dict[id] = w3lib.html.remove_tags(str(category))
                details['categories'].append(dict)

            # prices
            details['price'] = {}
            details['price']['initial'] = 0
            details['price']['final'] = int(leftcol.select('.discount_block')[0].get('data-price-final'))
            details['price']['discount_percent'] = int(
                re.findall(r'\d+', leftcol.select('.bundle_base_discount')[0].text)[0])
            details['price']['initial_formatted'] = leftcol.select('.bundle_final_package_price')[0].text
            details['price']['final_formatted'] = leftcol.select('.discount_final_price')[0].text
            details['price']['initial'] = math.ceil(
                (details['price']['final'] / 100) / ((100 - details['price']['discount_percent']) / 100)) * 100

            # package items
            details['package_item'] = []
            for item in leftcol.findAll('div', {'class': 'tab_item'}):
                package_item = {}
                package_item['name'] = item.select('.tab_item_name')[0].text
                if item.get('data-ds-packageid'):
                    package_item['packageid'] = item.get('data-ds-packageid')
                    package_item['appid'] = item.get('data-ds-appid').split(",")
                    for i in range(len(package_item['appid'])):
                        package_item['appid'][i] = int(package_item['appid'][i])
                else:
                    package_item['appid'] = int(item.get('data-ds-appid'))

                package_item['app_link'] = item.select('.tab_item_overlay')[0].get('href')
                package_item['app_image'] = item.select('.tab_item_cap_img')[0].get('src')

                # app prices
                package_item['app_price'] = {}
                package_item['app_price']['final'] = int(item.select('.discount_block')[0].get('data-price-final'))
                package_item['app_price']['final_formatted'] = item.select('.discount_final_price')[0].text

                # platforms
                package_item['platforms'] = {}
                package_item['platforms']['windows'] = True if item.select('.win') else False
                package_item['platforms']['mac'] = True if item.select('.mac') else False
                package_item['platforms']['linux'] = True if item.select('.linux') else False

                # categories
                package_item['categories'] = item.select('.tab_item_details')[0].text.strip().split(",")

                details['package_item'].append(package_item)

            data[appid]['data'] = details

        # if game exists
        if data[appid]['success']:
            self.data = data[appid]['data']
            return self.data
        else:
            return None

    # populate internal data dictionary
    def _populate(self, appid):
        if not self.data or appid != self.appid or self.appid == 0:
            self.getRaw(appid)

    # returns name
    def getName(self, appid):
        self._populate(appid)
        return self.data['name'] if 'name' in self.data else None

    # returns header image
    def getHeaderImage(self, appid):
        self._populate(appid)
        return self.data['header_image'] if 'header_image' in self.data else None

    # returns bundle description
    def getBundleDescription(self, appid):
        self._populate(appid)
        return self.data['bundle_description'] if 'bundle_description' in self.data else None

    # returns genre
    def getGenres(self, appid):
        self._populate(appid)
        return self.data['genres'] if 'genres' in self.data else None

    # returns developer
    def getDevelopers(self, appid):
        self._populate(appid)
        return self.data['developers'] if 'developers' in self.data else None

    # returns publisher
    def getPublishers(self, appid):
        self._populate(appid)
        return self.data['publishers'] if 'publishers' in self.data else None

    # returns franchise
    def getFranchise(self, appid):
        self._populate(appid)
        return self.data['franchise'] if 'franchise' in self.data else None

    # returns languages
    def getLanguages(self, appid):
        self._populate(appid)
        return self.data['languages'] if 'languages' in self.data else None

    # returns drm
    def getDRM(self, appid):
        self._populate(appid)
        return self.data['drm'] if 'drm' in self.data else None

    # returns categories
    def getCategories(self, appid):
        self._populate(appid)
        return self.data['categories'] if 'categories' in self.data else None

    # returns price
    def getPrice(self, appid):
        self._populate(appid)
        return self.data['price'] if 'price' in self.data else None

    # returns package items
    def getPackageItem(self, appid):
        self._populate(appid)
        return self.data['package_item'] if 'package_item' in self.data else None



"""Main module."""
from .errors import InvalidArgument
from .errors import Errors
from .misc import FuzzySearch
from .app import App
from .package import Package
from .bundle import Bundle


class SteamStoreFront:
    """
    :param appid: application id
    :param category: application category
    :param name: application name
    :param url: store url
    :type appid: integer or numeric string
    :type category: string
    :type name: string
    :type url: string

    .. note::

        one of the 3 parameters is required

        - appid
        - name
        - url

        category defaults to "app"

    :raise InvalidArgument: See error message for explanation. Additionally error can be looked up for argument and type for error code.

    :rtype: None if key not found.
    """

    appid = None
    category = "app"
    publisher = None
    app = None
    bundle = None
    package = None
    Errors = Errors

    # populate
    def _populate(self, **kwargs):
        # check if arguments were passed
        if kwargs:
            # check which arguments were passed
            # check if appid was passed
            if "appid" in kwargs:
                # check if appid is int
                if isinstance(kwargs["appid"], int):
                    self.appid = str(kwargs["appid"])

                # check if appid is numeric string
                elif isinstance(kwargs["appid"], str) and kwargs["appid"].isnumeric():
                    self.appid = kwargs["appid"]

                # appid is not integer, throw error
                else:
                    raise InvalidArgument("Appid must be integer or numeric string.", kwargs, Errors.InvalidAppId)

                # appid is valid, now assign category
                # default category is app
                if self.appid:
                    self.category = 'app'

                    # valid categories
                    accepted = ('sub', 'package', 'app', 'bundle')

                    # check if passed category is valid
                    if "category" in kwargs:
                        if kwargs["category"] in accepted:
                            self.category = kwargs["category"]
                        else:
                            raise InvalidArgument("Invalid category, {}.".format(kwargs['category']), kwargs, Errors.InvalidCategory)

            # name was passed
            if "name" in kwargs:
                # query for appid
                obj = FuzzySearch()
                self.appid = obj.getAppID(kwargs["name"])
                self.category = 'app'
                if not self.appid:
                    raise InvalidArgument("App not found for this game {}".format(kwargs['name']), kwargs, Errors.InvalidName)

            # url was passed
            if "url" in kwargs:
                url = ''

                # check if url is valid
                valid = ('https://store.steampowered.com', 'store.steampowered.com')
                if kwargs["url"].startswith(valid):
                    if kwargs["url"].startswith(valid[1]):
                        url = "https://" + kwargs["url"]
                    else:
                        url = kwargs["url"]

                    # if url is valid get category and appid
                    self.category = kwargs["url"].split("/")[3]
                    if self.category == "publisher":
                        self.publisher = kwargs["url"].split("/")[4]
                    else:
                        self.appid = kwargs["url"].split("/")[4]

        if "init" not in kwargs:
            # no arugments were passed or details are not filled
            if not self.appid:
                raise InvalidArgument("No identifier was passed, atleast one of appid, url, name is required.", kwargs, Errors.NoArgumentPassed)

        else:
            self.app = App()
            self.bundle = Bundle()
            self.package = Package()

    def __init__(self, **kwargs):
        # populate
        kwargs["init"] = True
        self._populate(**kwargs)
        app = App()
        bundle = Bundle()
        package = Package()

    def getRaw(self, **kwargs):
        """
            getRaw(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getRaw(appid=appid, category=category, name=name, url=url)

            - supported categories = [app, sub, bundle]            

            :return: returns raw json data
            :rtype: dictionary
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getRaw(self.appid)

        # get data for packages
        elif self.category == "sub" or self.category == "package":
            return self.package.getRaw(self.appid)

        # get data for bundles
        elif self.category == "bundle":
            return self.bundle.getRaw(self.appid)

        else:
            raise InvalidArgument("No app with {} appid.".format(self.appid), kwargs, Errors.InvalidAppId)

    # returns store url
    def getLink(self, **kwargs):
        """
            getLink(appid=appid, category=category, name=name, url=url)

            .. code-block:: python
                getLink(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app, sub, bundle]
            
            :return: returns store url
            :rtype: string
        """

        if self.category == 'app':
            return 'https://store.steampowered.com/app/' + self.appid
        elif self.category == 'sub' or self.category == 'package':
            return 'https://store.steampowered.com/sub/' + self.appid
        elif self.category == 'bundle':
            return 'https://store.steampowered.com/bundle/' + self.appid

    # function to return price
    # currency only works with app and sub|package
    def getPrice(self, **kwargs):
        """
            getPrice(appid=appid, category=category, name=name, url=url, currency=currency)

            .. code-block:: python

                getPrice(appid=appid, category=category, name=name, url=url, currency=currency)
            
            - supported categories = [app, sub, bundle]
            - supported categories with currency = [app, sub]
            - currency should be in 2 letters, eg:- USD : us, INR : in, etc
            
            :return: returns price
            :rtype: dictionary
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            if "currency" in kwargs:
                return self.app.getPriceInCurrency(self.appid, kwargs["currency"])
            return self.app.getPriceOverview(self.appid)

        # get data for packages
        elif self.category == "sub" or self.category == "package":
            if "currency" in kwargs:
                return self.package.getPriceInCurrency(self.appid, kwargs["currency"])
            return self.package.getPrice(self.appid)

        # get data for bundles
        elif self.category == "bundle":
            return self.bundle.getPrice(self.appid)

    # returns name
    def getName(self, **kwargs):
        """
            getName(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getName(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app, sub, bundle]
            
            :return: returns name
            :rtype: str
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getName(self.appid)

        # get data for packages
        elif self.category == "sub" or self.category == "package":
            return self.package.getName(self.appid)

        # get data for bundles
        elif self.category == "bundle":
            return self.bundle.getName(self.appid)

    # returns required age
    def getRequiredAge(self, **kwargs):
        """
            getRequiredAge(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getRequiredAge(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns required age
            :rtype: int
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getRequiredAge(self.appid)

        else:
            return None

    # returns is free
    def getIsFree(self, **kwargs):
        """
            getIsFree(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getIsFree(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns is game free
            :rtype: boolean
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getIsFree(self.appid)

        # get data for bundles
        else:
            return None

    # returns controller support
    def getControllerSupport(self, **kwargs):
        """
            getControllerSupport(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getControllerSupport(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app, sub]
            
            :return: returns controller support
            :rtype: str
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getControllerSupport(self.appid)

        # get data for packages
        elif self.category == "sub" or self.category == "package":
            return self.package.getController(self.appid)

        else:
            return None

    # returns dlc
    def getDLC(self, **kwargs):
        """
            getDLC(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getDLC(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns dlc list
            :rtype: list
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getDLC(self.appid)

        else:
            return None

    # returns detailed description
    # format = raw (default) | normal
    def getDescription(self, **kwargs):
        """
            getDescription(appid=appid, category=category, name=name, url=url, format=format)

            .. code-block:: python

                getDescription(appid=appid, category=category, name=name, url=url, format=format)
            
            - supported categories = [app, bundle]
            - supported categories with format = [app]
            - supported format = [raw(default), normal]

            - normal removes all html tags
            
            :return: returns description
            :rtype: str
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            if "short" in kwargs:
                return self.app.getShortDescription(self.appid)
            if "format" in kwargs:
                return self.app.getDetailedDescription(self.appid, kwargs["format"])
            return self.app.getDetailedDescription(self.appid, "raw")

        # get data for bundles
        elif self.category == "bundle":
            return self.bundle.getBundleDescription(self.appid)

        else:
            return None

    # returns about the game
    # format = raw (default) | normal
    def getAboutTheGame(self, **kwargs):
        """
            getAboutTheGame(appid=appid, category=category, name=name, url=url, format=format)

            .. code-block:: python

                getAboutTheGame(appid=appid, category=category, name=name, url=url, format=format)
            
            - supported categories = [app]
            - supported categories with format = [app]
            - supported format = [raw(default), normal]

            - normal removes all html tags
            
            :return: returns about the game
            :rtype: str
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            if "format" in kwargs:
                return self.app.getAboutTheGame(self.appid, kwargs["format"])
            return self.app.getAboutTheGame(self.appid, "raw")

        else:
            return None

    # returns short description
    def getShortDescription(self, **kwargs):
        """
            getShortDescription(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getShortDescription(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns short description
            :rtype: str
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getShortDescription(self.appid)

        else:
            return None

    # returns fullgame
    # IDK what this is
    def getFullgame(self, **kwargs):
        """
            getFullgame(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getFullgame(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns ?
            :rtype: ?
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getFullgame(self.appid)

        else:
            return None

    # returns supported languages
    # format = raw (default) | normal | list
    # format is only for app category
    def getSupportedLanguages(self, **kwargs):
        """
            getSupportedLanguages(appid=appid, category=category, name=name, url=url, format=format)

            .. code-block:: python

                getSupportedLanguages(appid=appid, category=category, name=name, url=url, format=format)
            
            - supported categories = [app, bundle]
            - supported categories with format = [app]
            - supported format = [raw(default), normal, list]

            - normal removes all html tags
            - list returns a list
            
            :return: returns supported language
            :rtype: str or list
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            if "format" in kwargs:
                return self.app.getSupportedLanguages(self.appid, kwargs["format"])
            else:
                return self.app.getSupportedLanguages(self.appid, "raw")

        # get data for bundles
        elif self.category == "bundle":
            return self.bundle.getLanguages(self.appid)

        else:
            return None

    # returns reviews
    # format = raw (default) | list
    '''
        list of dictinaries
        [
            {'review': '', 'score': '', 'link': '', 'reviewer'},
            {'review': '', 'score': '', 'link': '', 'reviewer'}
        ]
    '''

    def getReviews(self, **kwargs):
        """
            getReviews(appid=appid, category=category, name=name, url=url, format=format)

            .. code-block:: python

                getReviews(appid=appid, category=category, name=name, url=url, format=format)
            
            - supported categories = [app]
            - supported categories with format = [app]
            - supported format = [raw(default), list]
            
            :return: returns reviews
            :rtype: str or list
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            if "format" in kwargs:
                return self.app.getReviews(self.appid, kwargs["format"])
            else:
                return self.app.getReviews(self.appid, "raw")

        else:
            return None

    # returns header image
    def getHeaderImage(self, **kwargs):
        """
            getHeaderImage(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getHeaderImage(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app, sub, bundle]
            
            :return: returns header image
            :rtype: str
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getHeaderImage(self.appid)

        # get data for packages
        elif self.category == "sub" or self.category == "package":
            return self.package.getHeaderImage(self.appid)

        # get data for bundles
        elif self.category == "bundle":
            return self.bundle.getHeaderImage(self.appid)

        else:
            return None

    # returns website
    def getWebsite(self, **kwargs):
        """
            getWebsite(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getWebsite(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns 
            :rtype: 
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getWebsite(self.appid)

        else:
            return None

    # returns pc requirements
    # format = raw (default) | dict
    def getPCRequirements(self, **kwargs):
        """
            getPCRequirements(appid=appid, category=category, name=name, url=url, format=format)

            .. code-block:: python

                getPCRequirements(appid=appid, category=category, name=name, url=url, format=format)
            
            - supported categories = [app]
            - supported categories with format = [app]
            - supported format = [raw(default), dict]
            
            :return: returns pc requirements
            :rtype: str or dict
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            if "format" in kwargs:
                return self.app.getPCRequirements(self.appid, kwargs["format"])
            else:
                return self.app.getPCRequirements(self.appid, "raw", "raw")
        else:
            return None

    # returns mac requirements
    # format = raw (default) | dict
    def getMacRequirements(self, **kwargs):
        """
            getMacRequirements(appid=appid, category=category, name=name, url=url, format=format)

            .. code-block:: python

                getMacRequirements(appid=appid, category=category, name=name, url=url, format=format)
            
            - supported categories = [app]
            - supported categories with format = [app]
            - supported format = [raw(default), dict]
            
            :return: returns mac requirements
            :rtype: str or dict
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            if "format" in kwargs:
                return self.app.getMacRequirements(self.appid, kwargs["format"])
            else:
                return self.app.getMacRequirements(self.appid, "raw")
        else:
            return None

    # returns linux requirements
    # format = raw (default) | dict
    def getLinuxRequirements(self, **kwargs):
        """
            getLinuxRequirements(appid=appid, category=category, name=name, url=url, format=format)

            .. code-block:: python

                getLinuxRequirements(appid=appid, category=category, name=name, url=url, format=format)
            
            - supported categories = [app]
            - supported categories with format = [app]
            - supported format = [raw(default), dict]
            
            :return: returns linux requirements
            :rtype: str or dict
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            if "format" in kwargs:
                return self.app.getMacRequirements(self.appid, kwargs["format"])
            else:
                return self.app.getMacRequirements(self.appid, "raw")
        else:
            return None

    # returns legal notice
    def getLegalNotice(self, **kwargs):
        """
            getLegalNotice(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getLegalNotice(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns legal notice
            :rtype: str
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getLegalNotice(self.appid)

        else:
            return None

    # returns developers
    def getDevelopers(self, **kwargs):
        """
            getDevelopers(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getDevelopers(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app, bundle]
            
            :return: returns developers
            :rtype: list
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getDevelopers(self.appid)

        # get data for bundles
        elif self.category == "bundle":
            return self.bundle.getDevelopers(self.appid)

        else:
            return None

    # returns publishers
    def getPublishers(self, **kwargs):
        """
            getPublishers(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getPublishers(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app, bundle]
            
            :return: returns publishers
            :rtype: list
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getPublishers(self.appid)

        # get data for bundles
        elif self.category == "bundle":
            return self.bundle.getPublishers(self.appid)

        else:
            return None

    # returns demos
    def getDemos(self, **kwargs):
        """
            getDemos(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getDemos(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns demos
            :rtype: ?
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getDemos(self.appid)

        else:
            return None

    # returns packages
    def getPackages(self, **kwargs):
        """
            getPackages(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getPackages(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns packages
            :rtype: list
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getPackages(self.appid)

        else:
            return None

    # returns package group
    def getPackageGroup(self, **kwargs):
        """
            getPackageGroup(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getPackageGroup(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns package groups
            :rtype: list
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getPackageGroup(self.appid)

        else:
            return None

    # returns platforms
    def getPlatforms(self, **kwargs):
        """
            getPlatforms(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getPlatforms(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app, sub]
            
            :return: returns platforms
            :rtype: dict
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getPlatforms(self.appid)

        # get data for packages
        elif self.category == "sub" or self.category == "package":
            return self.package.getPlatforms(self.appid)

        else:
            return None

    # returns metacritic score
    def getMetacritic(self, **kwargs):
        """
            getMetacritic(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getMetacritic(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns metacritc score
            :rtype: dict
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getMetacritic(self.appid)

        else:
            return None

    # returns categories
    def getCategories(self, **kwargs):
        """
            getCategories(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getCategories(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app, bundle]
            
            :return: returns categories
            :rtype: list
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getCategories(self.appid)

        # get data for bundles
        elif self.category == "bundle":
            return self.bundle.getCategories(self.appid)

        else:
            return None

    # returns genres
    def getGenres(self, **kwargs):
        """
            getGenres(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getGenres(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app, bundle]
            
            :return: returns genres
            :rtype: list
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getGenres(self.appid)

        # get data for bundles
        elif self.category == "bundle":
            return self.bundle.getGenres(self.appid)

        else:
            return None

    # returns screenshots
    def getScreenshots(self, **kwargs):
        """
            getScreenshots(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getScreenshots(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns screenshots
            :rtype: list
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getScreenshots(self.appid)

        else:
            return None

    # returns movies
    def getMovies(self, **kwargs):
        """
            getMovies(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getMovies(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns movies
            :rtype: list
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getMovies(self.appid)

        else:
            return None

    # returns recommendations
    def getRecommendations(self, **kwargs):
        """
            getRecommendations(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getRecommendations(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns recommendations
            :rtype: dict
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getRecommendations(self.appid)

        else:
            return None

    # returns achievements
    def getAchievements(self, **kwargs):
        """
            getAchievements(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getAchievements(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns achievements list
            :rtype: list
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getAchievements(self.appid)

        else:
            return None

    # returns release date
    def getReleaseDate(self, **kwargs):
        """
            getReleaseDate(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getReleaseDate(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app, sub]
            
            :return: returns release date
            :rtype: dict
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getReleaseDate(self.appid)

        # get data for packages
        elif self.category == "sub" or self.category == "package":
            return self.package.getReleaseDate(self.appid)

        else:
            return None

            # returns ratings

    def getRatings(self, **kwargs):
        """
            getRatings(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getRatings(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns (review_score, rating, review_summary)
            :rtype: tuple
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getRatings(self.appid)

        else:
            return None

    # returns release date
    def getSupportInfo(self, **kwargs):
        """
            getSupportInfo(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getSupportInfo(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns suport info
            :rtype: dict
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getSupportInfo(self.appid)

        else:
            return None

    # returns release date
    def getBackground(self, **kwargs):
        """
            getBackground(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getBackground(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns background image
            :rtype: str
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getBackground(self.appid)

        else:
            return None

    # returns release date
    def getContentDescriptors(self, **kwargs):
        """
            getContentDescriptors(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getContentDescriptors(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [app]
            
            :return: returns content descriptors
            :rtype: dict
        """

        # store data
        self._populate(**kwargs)

        # get data for app
        if self.category == "app":
            return self.app.getContentDescriptors(self.appid)

        else:
            return None

    '''
        Bundle only functions
    '''

    # returns franchise
    def getFranchise(self, **kwargs):
        """
            getFranchise(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getFranchise(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [bundle]
            
            :return: returns franchise
            :rtype: list
        """

        # store data
        self._populate(**kwargs)

        if self.category == "bundle":
            return self.bundle.getFranchise(self.appid)

        else:
            return None

    # returns drm
    def getDRM(self, **kwargs):
        """
            getDRM(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getDRM(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [bundle]
            
            :return: returns drm
            :rtype: list
        """

        # store data
        self._populate(**kwargs)

        if self.category == "bundle":
            return self.bundle.getDRM(self.appid)

        else:
            return None

    # returns package items
    def getPackageItem(self, **kwargs):
        """
            getPackageItem(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getPackageItem(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [bundle]
            
            :return: returns package item (apps and packages)
            :rtype: list
        """

        # store data
        self._populate(**kwargs)

        if self.category == "bundle":
            return self.bundle.getPackageItem(self.appid)

        else:
            return None

    '''
        Package only functions
    '''

    # returns required age
    def getPageImage(self, **kwargs):
        """
            getPageImage(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getPageImage(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [sub]
            
            :return: returns page image
            :rtype: str
        """

        # store data
        self._populate(**kwargs)

        if self.category == "sub" or self.category == "package":
            return self.package.getPageImage(self.appid)

        else:
            return None

    # returns small logo
    def getSmallLogo(self, **kwargs):
        """
            getSmallLogo(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getSmallLogo(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [sub]
            
            :return: returns small logo
            :rtype: str
        """

        # store data
        self._populate(**kwargs)

        if self.category == "sub" or self.category == "package":
            return self.package.getSmallLogo(self.appid)

        else:
            return None

    # returns apps
    def getApps(self, **kwargs):
        """
            getApps(appid=appid, category=category, name=name, url=url)

            .. code-block:: python

                getApps(appid=appid, category=category, name=name, url=url)
            
            - supported categories = [sub]
            
            :return: returns apps
            :rtype: list
        """

        # store data
        self._populate(**kwargs)

        if self.category == "sub" or self.category == "package":
            return self.package.getApps(self.appid)

        else:
            return None



#!/usr/bin/env python

"""Tests for `steamstorefront` package."""

import pytest

from click.testing import CliRunner

from steamstorefront import SteamStoreFront
from steamstorefront import cli

appid = 58375

@pytest.fixture
def app():
    data = SteamStoreFront()
    return data

# returns raw data
def testGetRaw(app):
    data = app.getRaw(appid=appid, category='package')
    assert data['name'] == "Lara Croft and the Temple of Osiris - Season Pass Only"

 # returns name
def testGetName(app):
    data = app.getName(appid=appid, category='package')
    test = (type(data) == type(str()) and data == "Lara Croft and the Temple of Osiris - Season Pass Only")
    assert test == True

# returns required age
def testGetPageImage(app):
    data = app.getPageImage(appid=appid, category='package')
    test = (type(data) == type(str()) and ".jpg" in data)
    assert test == True

# returns is free
def testGetHeaderImage(app):
    data = app.getHeaderImage(appid=appid, category='package')
    test = (type(data) == type(str()) and ".jpg" in data)
    assert test == True

# returns controller support
def testGetSmallLogo(app):
    data = app.getSmallLogo(appid=appid, category='package')
    test = (type(data) == type(str()) and ".jpg" in data)
    assert test == True

# returns dlc
def testGetApps(app):
    data = app.getApps(appid=appid, category='package')
    test = (type(data) == type(list()) and data[0]['id'] == 318874)
    assert test == True

# returns detailed description
def testGetPrice(app):
    data1 = app.getPrice(appid=appid, category='package')
    data2 = app.getPrice(appid=appid, category='package', currency='in')
    test1 = (type(data1) == type(dict()))
    test2 = (type(data2) == type(dict()))
    assert test1 == True
    assert test2 == True

# returns short description
def testGetPlatforms(app):
    data = app.getPlatforms(appid=appid, category='package')
    test = (type(data) == type(dict()) and data['mac'] == False)
    assert test == True

# returns fullgame
def testGetController(app):
    data = app.getControllerSupport(appid=appid, category='package')
    test = (type(data) == type(dict()) and data['full_gamepad'] == True)
    assert test == True

# returns header image
def testGetReleaseDate(app):
    data = app.getReleaseDate(appid=appid, category='package')
    test = (type(data) == type(dict()) and data['coming_soon'] == False)
    assert test == True


"""Top-level package for Steam Store Front."""

__author__ = """Santosh Panna"""
__email__ = 'santoshpanna.sp@gmail.com'
__version__ = '0.0.3.5'

import builtins

from .steamstorefront import SteamStoreFront
from .errors import InvalidArgument

builtins.InvalidArgument = InvalidArgument



#!/usr/bin/env python

"""Tests for `steamstorefront` package."""

import pytest

from click.testing import CliRunner

from steamstorefront import SteamStoreFront
from steamstorefront import cli

# Tomb Raider Collection
appid = 2823

@pytest.fixture
def app():
    data = SteamStoreFront()
    return data

# returns raw data
def testGetRaw(app):
    data = app.getRaw(appid=appid, category='bundle')
    assert data['bundle_id'] == str(appid)

# returns name
def testGetName(app):
    data = app.getName(appid=appid, category='bundle')
    test = (data == "Tomb Raider Collection")
    assert test == True

# returns header image
def testGetHeaderImage(app):
    data = app.getHeaderImage(appid=appid, category='bundle')
    test = (".jpg" in data)
    assert test == True

# returns bundle description
def testGetBundleDescription(app):
    data = app.getDescription(appid=appid, category='bundle')
    test = (type(data) == type(str()))
    assert test == True

# returns genre
def testGetGenres(app):
    data = app.getGenres(appid=appid, category='bundle')
    test = (type(data) == type(list()) and "Action" in data)
    assert test == True

# returns developer
def testGetDevelopers(app):
    data = app.getDevelopers(appid=appid, category='bundle')
    test = (type(data) == type(list()) and "Crystal Dynamics" in data)
    assert test == True

# returns publisher
def testGetPublishers(app):
    data = app.getPublishers(appid=appid, category='bundle')
    test = (type(data) == type(list()) and "Square Enix" in data)
    assert test == True

# returns franchise
def testGetFranchise(app):
    data = app.getFranchise(appid=appid, category='bundle')
    test = (type(data) == type(list()) and "Tomb Raider" in data)
    assert test == True

# returns languages
def testGetLanguages(app):
    data = app.getSupportedLanguages(appid=appid, category='bundle')
    test = (type(data) == type(list()) and "English" in data)
    assert test == True

# returns drm
def testGetDRM(app):
    data = app.getDRM(appid=appid, category='bundle')
    test = (type(data) == type(str()))
    assert test == True

# returns categories
def testGetCategories(app):
    data = app.getCategories(appid=appid, category='bundle')
    test = (type(data) == type(list()) and data[0][2] == "Single-player")
    assert test == True

# returns price
def testGetPrice(app):
    data = app.getPrice(appid=appid, category='bundle')
    test = (type(data) == type(dict()))
    assert test == True

# returns package items
def testGetPackageItem(app):
    data = app.getPackageItem(appid=appid, category='bundle')
    test = (type(data) == type(list()) and data[0]['name'] == "Tomb Raider: Legend")
    assert test == True


import requests


class Package:
    api_url = "https://store.steampowered.com/api/packagedetails?packageids="
    data = {}
    appid = 0

    '''
        DATA
        App ID:
            sucess
            data:
                name
                page_image
                header_image (optional)
                small_logo
                apps
                price
                platforms
                controller
                release_date
    '''

    # returns raw data
    def getRaw(self, appid):
        # caching appid
        self.appid = appid

        # request the data
        json = requests.get(self.api_url + appid).json()

        # if game exists
        if json[appid]['success']:
            self.data = json[appid]['data']
            return self.data
        else:
            return None

    # populate internal data dictionary
    def _populate(self, appid):
        if not self.data or appid != self.appid or self.appid == 0:
            self.getRaw(appid)

    # returns name
    def getName(self, appid):
        self._populate(appid)
        return self.data['name'] if 'name' in self.data else None

    # returns required age
    def getPageImage(self, appid):
        self._populate(appid)
        return self.data['page_image'] if 'page_image' in self.data else None

    # returns is free
    def getHeaderImage(self, appid):
        self._populate(appid)
        return self.data['header_image'] if 'header_image' in self.data else None

    # returns controller support
    def getSmallLogo(self, appid):
        self._populate(appid)
        return self.data['small_logo'] if 'small_logo' in self.data else None

    # returns dlc
    def getApps(self, appid):
        self._populate(appid)
        return self.data['apps'] if 'apps' in self.data else None

    # returns detailed description
    def getPrice(self, appid):
        self._populate(appid)
        return self.data['price'] if 'price' in self.data else None

    # returns short description
    def getPlatforms(self, appid):
        self._populate(appid)
        return self.data['platforms'] if 'platforms' in self.data else None

    # returns fullgame
    def getController(self, appid):
        self._populate(appid)
        return self.data['controller'] if 'controller' in self.data else None

    # returns header image
    def getReleaseDate(self, appid):
        self._populate(appid)
        return self.data['release_date'] if 'release_date' in self.data else None

    # returns price in specific currency
    def getPriceInCurrency(self, appid, currency):
        req = requests.get(self.api_url + appid + "&cc=" + currency).json()

        if req[appid]["data"]:
            return req[appid]["data"]["price"]
        else:
            return None



import enum


class SteamStoreFront(Exception):
    pass


class Errors(enum.Enum):
    NoArgumentPassed = 0
    InvalidCategory = 1
    InvalidUrl = 2
    InvalidName = 3
    InvalidAppId = 4


class InvalidArgument(SteamStoreFront):
    def __init__(self, message, error, type_e):
        super().__init__(message)
        self.error = error
        self.type = type_e



#!/usr/bin/env python

"""Tests for `steamstorefront` package."""

import pytest

from click.testing import CliRunner

from steamstorefront import SteamStoreFront
from steamstorefront import cli

# Tomb Raider Reboot
appid = 203160

@pytest.fixture
def app():
    data = SteamStoreFront()
    return data


# returns raw data
def testGetRaw(app):
    data = app.getRaw(appid=appid, category='app')
    assert data['steam_appid'] == 203160

# returns name
def testGetName(app):
    assert app.getName(appid=appid, category='app') == "Tomb Raider"

# returns required age
def testGetRequiredAge(app):
    assert type(app.getRequiredAge(appid=appid, category='app')) == type(str())

# returns is free
def testGetIsFree(app):
    assert app.getIsFree(appid=appid, category='app') == False

# returns controller support
def testGetControllerSupport(app):
    assert app.getControllerSupport(appid=appid, category='app') == "full"

# returns dlc
def testGetDLC(app):
    data = app.getDLC(appid=appid, category='app')
    test = (type(data) == type(list()) and 208791 in data)
    assert test == True

# returns detailed description
# format = raw (default) | normal
def testGetDescription(app):
    assert type(app.getDescription(appid=appid, category='app', format='raw')) == type(str())
    assert type(app.getDescription(appid=appid, category='app', format='normal')) == type(str())

    assert type(app.getDescription(appid=appid, category='app', short=True, format='raw')) == type(str())
    assert type(app.getDescription(appid=appid, category='app', short=True, format='normal')) == type(str())

# returns about the game
# format = raw (default) | normal
def testGetAboutTheGame(app):
    assert type(app.getControllerSupport(appid=appid, category='app', format='raw')) == type(str())
    assert type(app.getControllerSupport(appid=appid, category='app', format='normal')) == type(str())

# returns fullgame
# IDK what this is
def testGetFullgame(app):
    assert app.getFullgame(appid=appid, category='app') == None

# returns supported languages
# format = raw (default) | normal | list
def testGetSupportedLanguages(app):
    data1 = app.getSupportedLanguages(appid=appid, category='app')
    data2 = app.getSupportedLanguages(appid=appid, category='app', format="normal")
    data3 = app.getSupportedLanguages(appid=appid, category='app', format="list")
    langtest1 = ("English" in data1 and type(str()) == type(data1) and "<strong>" in data1)
    langtest2 = ("English" in data2 and type(str()) == type(data2) and "<strong>" not in data2)
    langtest3 = ("English" in data3 and type(list()) == type(data3))

    assert langtest1 == True
    assert langtest2 == True
    assert langtest3 == True

# returns reviews
def testGetReviews(app):
    assert type(app.getReviews(appid=appid, category='app', format='raw')) == type(str())
    assert type(app.getReviews(appid=appid, category='app', format='list')) == type(list())

# returns header image
def testGetHeaderImage(app):
    test = ".jpg" in app.getHeaderImage(appid=appid, category='app')
    assert test == True

# returns website
def testGetWebsite(app):
    assert app.getWebsite(appid=appid, category='app') == "http://www.tombraider.com"

# returns pc requirements
# format = raw (default) | dict
def testGetPCRequirements(app):
    data1 = app.getPCRequirements(appid=appid, category='app', format='raw')
    data2 = app.getPCRequirements(appid=appid, category='app', format='dict')
    test1 = (data1["minimum"].startswith("<strong>") and type(data1["minimum"]) == type(str()))
    test2 = ("OS" in data2["minimum"] and type(data2["minimum"]) == type(dict()))
    assert test1 == True
    assert test2 == True

# returns mac requirements
# format = raw (default) | dict
def testGetMacRequirements(app):
    data1 = app.getMacRequirements(appid=appid, category='app', format='raw')
    data2 = app.getMacRequirements(appid=appid, category='app', format='dict')
    test1 = (data1["minimum"].startswith("<strong>") and type(data1["minimum"]) == type(str()))
    test2 = ("OS" in data2["minimum"] and type(data2["minimum"]) == type(dict()))
    assert test1 == True
    assert test2 == True

# returns linux requirements
# format = raw (default) | dict
def testGetLinuxRequirements(app):
    data1 = app.getLinuxRequirements(appid=appid, category='app', format='raw')
    data2 = app.getLinuxRequirements(appid=appid, category='app', format='dict')
    test1 = (data1["minimum"].startswith("<strong>") and type(data1["minimum"]) == type(str()))
    test2 = ("OS" in data2["minimum"] and type(data2["minimum"]) == type(dict()))
    assert test1 == True
    assert test2 == True

# returns legal notice
def testGetLegalNotice(app):
    assert type(app.getLegalNotice(appid=appid, category='app')) == type(str())

# returns developers
def testGetDevelopers(app):
    data = app.getDevelopers(appid=appid, category='app') 
    test1 = ("Crystal Dynamics" in data and type(data) == type(list()))
    assert test1 == True

# returns publishers
def testGetPublishers(app):
    data = app.getPublishers(appid=appid, category='app') 
    test1 = ("Square Enix" in data and type(data) == type(list()))
    assert test1 == True

# returns demos
def testGetDemos(app):
    assert app.getDemos(appid=appid, category='app') == None

# returns price overview
def testGetPrice(app):
    data1 = app.getPrice(appid=appid, category='app')
    data2 = app.getPrice(appid=appid, category='app', currency='in')
    test1 = (type(data1) == type(dict()) and type(data1["initial"]) == type(int()))
    test2 = (type(data2) == type(dict()) and data2["initial"] == 56500)
    assert test1 == True
    assert test2 == True

# returns packages
def testGetPackages(app):
    data = app.getPackages(appid=appid, category='app')
    test = (type(data) == type(list()) and 26016 in data)
    assert test == True

# returns package group
def testGetPackageGroup(app):
    data = app.getPackageGroup(appid=appid, category='app')
    test = (type(data) == type(list()) and data[0]["title"] == "Buy Tomb Raider")
    assert test == True

# returns platforms
def testGetPlatforms(app):
    data = app.getPlatforms(appid=appid, category='app')
    test = (type(data) == type(dict()) and data["windows"] == True)
    assert test == True

# returns metacritic score
def testGetMetacritic(app):
    data = app.getMetacritic(appid=appid, category='app')
    test = (type(data) == type(dict()) and type(data["score"]) == type(int()))
    assert test == True

# returns categories
def testGetCategories(app):
    data = app.getCategories(appid=appid, category='app')
    test = (type(data) == type(list()) and data[0]["id"] == 2)
    assert test == True

# returns genres
def testGetGenres(app):
    data = app.getGenres(appid=appid, category='app')
    test = (type(data) == type(list()) and data[0]["id"] == "1")
    assert test == True

# returns screenshots
def testGetScreenshots(app):
    data = app.getScreenshots(appid=appid, category='app')
    test = (type(data) == type(list()) and data[0]["id"] == 0)
    assert test == True

# returns movies
def testGetMovies(app):
    data = app.getMovies(appid=appid, category='app')
    test = (type(data) == type(list()))
    assert test == True

# returns recommendations
def testGetRecommendations(app):
    data = app.getRecommendations(appid=appid, category='app')
    test = (type(data) == type(dict()) and type(data["total"]) == type(int()))
    assert test == True

# returns achievements
def testGetAchievements(app):
    data = app.getAchievements(appid=appid, category='app')
    test = (type(data) == type(dict()) and data["total"] == 50)
    assert test == True

# returns release date
def testGetReleaseDate(app):
    data = app.getReleaseDate(appid=appid, category='app')
    test = (type(data) == type(dict()) and data["coming_soon"] == False)
    assert test == True

# returns ratings
def testGetRatings(app):
    data = app.getRatings(appid=appid, category='app')
    test = (type(data) == type(tuple()) and data[0] != 0)
    assert test == True

# returns release date
def testGetSupportInfo(app):
    data = app.getSupportInfo(appid=appid, category='app')
    test = (type(data) == type(dict()) and data["url"] == "http://support.na.square-enix.com/")
    assert test == True

# returns release date
def testGetBackground(app):
    data = app.getBackground(appid=appid, category='app')
    test = (type(data) == type(str()) and '.jpg' in data)
    assert test == True

# returns release date
def testGetContentDescriptors(app):
    data = app.getContentDescriptors(appid=appid, category='app')
    test = (type(data) == type(dict()) and type(data["ids"]) == type(list()))
    assert test == True

def test_command_line_interface():
    """Test the CLI."""
    runner = CliRunner()
    result = runner.invoke(cli.main)
    assert result.exit_code == 0
    assert 'steamstorefront.cli.main' in result.output
    help_result = runner.invoke(cli.main, ['--help'])
    assert help_result.exit_code == 0
    assert '--help  Show this message and exit.' in help_result.output


import requests, w3lib.html, math


class App:
    api_url = "https://store.steampowered.com/api/appdetails?appids="
    data = {}
    appid = 0

    def _getNormal(self, text):
        text = text.replace("\"", "").replace("<img src=", "").replace(" >", "")
        return text.replace("<br><br>", "\n")

    def _getNormalLanguage(self, text):
        text = w3lib.html.remove_tags(text)
        return text.replace("*", "").replace("languages with full audio support", "")

    # makes a list from text
    # iterates until text is empty 
    # gets key, value from html patterns
    def _getListReview(self, text):
        reviews = []
        while text != "":
            review = {}
            review['review'] = text[:text.find("<br>")]
            text = text[text.find("<br>") + 4:]
            review['score'] = text[:text.find(" ")]
            text = text[text.find(" ") + 12:]
            review['link'] = text[:text.find("\"")].replace("https://steamcommunity.com/linkfilter/?url=", "")
            text = text[text.find(">") + 1:]
            review['reviewer'] = text[:text.find("<")]
            text = text[text.find("<br>") + 8:]
            reviews.append(review)
        return reviews

    # makes a dict from text
    # iterates until text is empty
    # gets key, value from html patterns
    def _getDict(self, text):
        dict = {}

        text = w3lib.html.remove_tags(text, keep=('strong',))

        while text != "":
            key = text[:text.find("</strong>") - 1].replace("<strong>", "")
            text = text[text.find("</strong>") + 9:]
            requirement = text[:text.find("<strong>")]
            dict[key] = requirement
            if text.find(">") == -1:
                dict[key] = text
                text = ""
            else:
                text = text[text.find(">") + 1:]

        return dict

    def _getDictionaryRequirements(self, list):
        dict = {}
        if 'minimum' in list:
            dict['minimum'] = self._getDict(list['minimum'])
        if 'recommended' in list:
            dict['recommended'] = self._getDict(list['recommended'])

        return dict

    '''
        DATA
        App ID:
            sucess
            data:
                type - ["game", "dlc", "demo", "advertising", "mod", "video"]
                name
                steam_appid
                required_age
                is_free
                controller_support (optional)
                dlc (optional)
                detailed_description
                about_the_game
                short_description
                fullgame (optional)
                supported_languages
                reviews
                header_image
                website
                pc_requirements
                mac_requirements
                linux_requirements
                legal_notice (optional)
                developers (optional)
                publishers
                demos (optional)
                price_overview (optional)
                packages
                package_group
                platforms
                metacritic (optional)
                categories (optional)
                genres (optional)
                screenshots (optional)
                movies (optional)
                recommendations (optional)
                achievements (optional)
                release_date
                support_info
                background
                content_desriptors
    '''

    # returns raw data
    def getRaw(self, appid):
        # caching appid
        self.appid = appid

        # request the data
        json = requests.get(self.api_url + appid).json()

        # if game exists
        if json[appid]['success']:
            self.data = json[appid]['data']
            return self.data
        else:
            return None

    # populate internal data dictionary
    def _populate(self, appid):
        if not self.data or appid != self.appid or str(self.data['steam_appid']) != appid:
            self.getRaw(appid)

    # returns name
    def getName(self, appid):
        self._populate(appid)
        return self.data['name'] if 'name' in self.data else None

    # returns required age
    def getRequiredAge(self, appid):
        self._populate(appid)
        return self.data['required_age'] if 'required_age' in self.data else None

    # returns is free
    def getIsFree(self, appid):
        self._populate(appid)
        return self.data['is_free'] if 'is_free' in self.data else None

    # returns controller support
    def getControllerSupport(self, appid):
        self._populate(appid)
        return self.data['controller_support'] if 'controller_support' in self.data else None

    # returns dlc
    def getDLC(self, appid):
        self._populate(appid)
        return self.data['dlc'] if 'dlc' in self.data else None

    # returns detailed description
    # format = raw (default) | normal
    def getDetailedDescription(self, appid, format: str = None):
        self._populate(appid)

        # if normal remove just html tags, retain images link
        if format.lower() == "normal":
            return self._getNormal(self.data['detailed_description']) if 'detailed_description' in self.data else None
        else:
            return self.data['detailed_description'] if 'detailed_description' in self.data else None

    # returns about the game
    # format = raw (default) | normal
    def getAboutTheGame(self, appid, format: str = None):
        self._populate(appid)
        # if normal remove just html tags, retain images link
        if format.lower() == "normal":
            return self._getNormal(self.data['about_the_game']) if 'about_the_game' in self.data else None
        else:
            return self.data['about_the_game'] if 'about_the_game' in self.data else None

    # returns short description
    def getShortDescription(self, appid):
        self._populate(appid)
        return self.data['short_description'] if 'short_description' in self.data else None

    # returns fullgame
    # IDK what this is
    def getFullgame(self, appid):
        self._populate(appid)
        return self.data['fullgame'] if 'fullgame' in self.data else None

    # returns supported languages
    # format = raw (default) | normal | list
    def getSupportedLanguages(self, appid, format: str = None):
        self._populate(appid)
        if format.lower() == "normal":
            return self._getNormalLanguage(
                self.data['supported_languages']) if 'supported_languages' in self.data else None
        elif format.lower() == "list":
            return self._getNormalLanguage(self.data['supported_languages']).split(
                ", ") if 'supported_languages' in self.data else None
        else:
            return self.data['supported_languages'] if 'supported_languages' in self.data else None

    # returns reviews
    # format = raw (default) | list
    '''
        list of dictinaries
        [
            {'review': '', 'score': '', 'link': '', 'reviewer'},
            {'review': '', 'score': '', 'link': '', 'reviewer'}
        ]
    '''

    def getReviews(self, appid, format: str = None):
        self._populate(appid)
        if format.lower() == "list":
            return self._getListReview(self.data['reviews']) if 'reviews' in self.data else None
        else:
            return self.data['reviews'] if 'reviews' in self.data else None

    # returns header image
    def getHeaderImage(self, appid):
        self._populate(appid)
        return self.data['header_image'] if 'header_image' in self.data else None

    # returns website
    def getWebsite(self, appid):
        self._populate(appid)
        return self.data['website'] if 'website' in self.data else None

    # returns pc requirements
    # format = raw (default) | dict
    def getPCRequirements(self, appid, format: str = None):
        self._populate(appid)
        if format.lower() == "dict":
            return self._getDictionaryRequirements(
                self.data['pc_requirements']) if 'pc_requirements' in self.data else None
        else:
            return self.data['pc_requirements'] if 'pc_requirements' in self.data else None

    # returns mac requirements
    # format = raw (default) | dict
    def getMacRequirements(self, appid, format: str = None):
        self._populate(appid)

        if format.lower() == "dict":
            return self._getDictionaryRequirements(
                self.data['mac_requirements']) if 'mac_requirements' in self.data else None
        else:
            return self.data['mac_requirements'] if 'mac_requirements' in self.data else None

    # returns linux requirements
    # format = raw (default) | dict
    def getLinuxRequirements(self, appid, format: str = None):
        self._populate(appid)
        if format.lower() == "dict":
            return self._getDictionaryRequirements(
                self.data['linux_requirements']) if 'linux_requirements' in self.data else None
        else:
            return self.data['linux_requirements'] if 'linux_requirements' in self.data else None

    # returns legal notice
    def getLegalNotice(self, appid):
        self._populate(appid)
        return self.data['legal_notice'] if 'legal_notice' in self.data else None

    # returns developers
    def getDevelopers(self, appid):
        self._populate(appid)
        return self.data['developers'] if 'developers' in self.data else None

    # returns publishers
    def getPublishers(self, appid):
        self._populate(appid)
        return self.data['publishers'] if 'publishers' in self.data else None

    # returns demos
    def getDemos(self, appid):
        self._populate(appid)
        return self.data['demos'] if 'demos' in self.data else None

    # returns price overview
    def getPriceOverview(self, appid):
        self._populate(appid)
        return self.data['price_overview'] if 'price_overview' in self.data else None

    # returns packages
    def getPackages(self, appid):
        self._populate(appid)
        return self.data['packages'] if 'packages' in self.data else None

    # returns package group
    def getPackageGroup(self, appid):
        self._populate(appid)
        return self.data['package_groups'] if 'package_groups' in self.data else None

    # returns platforms
    def getPlatforms(self, appid):
        self._populate(appid)
        return self.data['platforms'] if 'platforms' in self.data else None

    # returns metacritic score
    def getMetacritic(self, appid):
        self._populate(appid)
        return self.data['metacritic'] if 'metacritic' in self.data else None

    # returns categories
    def getCategories(self, appid):
        self._populate(appid)
        return self.data['categories'] if 'categories' in self.data else None

    # returns genres
    def getGenres(self, appid):
        self._populate(appid)
        return self.data['genres'] if 'genres' in self.data else None

    # returns screenshots
    def getScreenshots(self, appid):
        self._populate(appid)
        return self.data['screenshots'] if 'screenshots' in self.data else None

    # returns movies
    def getMovies(self, appid):
        self._populate(appid)
        return self.data['movies'] if 'movies' in self.data else None

    # returns recommendations
    def getRecommendations(self, appid):
        self._populate(appid)
        return self.data['recommendations'] if 'recommendations' in self.data else None

    # returns achievements
    def getAchievements(self, appid):
        self._populate(appid)
        return self.data['achievements'] if 'achievements' in self.data else None

    # returns release date
    def getReleaseDate(self, appid):
        self._populate(appid)
        return self.data['release_date'] if 'release_date' in self.data else None

        # returns release date

    def getSupportInfo(self, appid):
        self._populate(appid)
        return self.data['support_info'] if 'support_info' in self.data else None

        # returns release date

    def getBackground(self, appid):
        self._populate(appid)
        return self.data['background'] if 'background' in self.data else None

        # returns release date

    def getContentDescriptors(self, appid):
        self._populate(appid)
        return self.data['content_descriptors'] if 'content_descriptors' in self.data else None

        # returns ratings

    def getRatings(self, appid):
        req = requests.get("https://store.steampowered.com/appreviews/" + appid + "?json=1").json()

        if req["query_summary"]:
            # Review Score = frac{Positive Reviews}{Total Reviews}
            try:
                review_score = req["query_summary"]["total_positive"] / req["query_summary"]["total_reviews"]
                # Rating = Review Score - (Review Score - 0.5)*2^{-log_{10}(Total Reviews + 1)})
                rating = review_score - (review_score - 0.5) * (
                            2 ** (-1 * math.log10(req["query_summary"]["total_reviews"] + 1)))
                return review_score * 100, rating * 100, req["query_summary"]
            except:
                return 0, 0, req["query_summary"]
        else:
            return 0, 0, req["query_summary"]

    # returns price in specific currency
    def getPriceInCurrency(self, appid, currency):
        req = requests.get(self.api_url + appid + "&cc=" + currency + "&filters=price_overview").json()

        if req[appid]["data"]:
            return req[appid]["data"]["price_overview"]
        else:
            return None
